#include "FileMaster.h"
#include <iostream>
#include <fstream>
#include <iomanip>
#include <deque>

void printMainMenu() {
	std::cout << "                     M E N U                     " << std::endl << std::endl
		<< "1. Create a file with a file stream" << std::endl
		<< "2. Create a file with stream iterators and GENERATE algorithm" << std::endl
		<< "3. Load data in a container with a file stream" << std::endl
		<< "4. Load data in a container with a stream iterator and COPY algorithm" << std::endl
		<< "5. Modify data (template function)" << std::endl
		<< "6. Modify data via container iterators" << std::endl
		<< "7. Modify data via TRANSFORM algorithm" << std::endl
		<< "8. Output data with a file stream" << std::endl
		<< "9. Output data via COPY algorithm and stream iterators" <<std::endl
		<< "e. Exit" << std::endl << std::endl
		<< "Waiting for input... " << std::endl;
}

int main() {
	std::string fileName = "Numbers.txt";
	std::string resultFile = "Result.txt";
	std::deque<int> cont;
	FileMaster fm;

	char input = 'q';
	while (input != 'e') {
		printMainMenu();
		std::cin >> input;
		std::cout << std::endl << std::endl;
		std::ofstream ofstr;
		std::ifstream ifstr;

		switch (input){
		case'1':
			ofstr.open(fileName);
			try {
				fm.createData(ofstr);
			}
			catch (std::runtime_error err) {
				std::cout << err.what() << std::endl;
			}
			std::cout << "File created!" << std::endl;
			ofstr.close();
			break;

		case'2':
			ofstr.open(fileName);
			fm.createData(std::ostream_iterator<int>(ofstr, " "));
			std::cout << "File created!" << std::endl;
			ofstr.close();
			break;

		case'3':
			ifstr.open(fileName);
			try {
				fm.loadData(cont, ifstr);
			}
			catch (std::runtime_error err) {
				std::cout << err.what() << std::endl;
			}
			
			std::cout << "Data is loaded in container!" << std::endl;
			ifstr.close();
			break;
			
		case'4':
			ifstr.open(fileName);
			if (!ifstr.is_open()) {
				std::cout << "File is not opened." << std::endl;
				break;
			}
			fm.loadData(cont, std::istream_iterator<int>(ifstr), std::istream_iterator<int>());
			std::cout << "Data is loaded in container!" << std::endl;
			ifstr.close();
			break;

		case'5':
			try {
				fm.modify(cont);
			}
			catch (std::exception ex) {
				std::cout << ex.what() << std::endl;
			}
			break;

		case'6':
			try {
				fm.modify(cont.begin(), cont.end());
			}
			catch (std::exception ex) {
				std::cout << ex.what() << std::endl;
			}
			break;

		case'7':
			ifstr.open(fileName);
			ofstr.open(resultFile);
			if (!ifstr.is_open() && !ofstr.is_open()) {
				std::cout << "File is not opened." << std::endl;
				break;
			}
			fm.modify(std::istream_iterator<int>(ifstr), std::istream_iterator<int>(), std::ostream_iterator<int>(ofstr, " "));
			ifstr.close();
			ofstr.close();


		case'8':
			ofstr.open(resultFile);
			try {
				fm.outputResult(cont, ofstr);
			}
			catch(std::exception ex){
				std::cout << ex.what() << std::endl;
			}
			ofstr.close();

		case'9':
			ofstr.open(resultFile);
			try {
				fm.outputResult(cont, std::ostream_iterator<int>(ofstr," "));
			}
			catch(std::exception ex){
				std::cout << ex.what() << std::endl;
			}
			ofstr.close();
		case'e':
			break;

		default:
			std::cout << "Wrong input." << std::endl;
		}
	}
	
	return 0;
}